@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018, 2019  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -34,6 +34,12 @@
 #include "debug.h"
 #endif
 
+#define strulen(a)                           \
+    (                                        \
+        STATIC(TYPEOF_IS_CHAR_UCHAR_PTR(a)), \
+        strlen((const char*) (a))            \
+    )
+
 // stev: important TODO: review all paired invocations of
 // 'POP(?!_N)|TOP(?:_N)?' with 'ENSURE' as to replace them
 // with new macro calls '(?:POP(?!_N)|TOP(?:_N)?)_AS'
@@ -131,6 +137,32 @@ static void* json_ast_allocate(
     return n;
 }
 
+#define JSON_AST_ALLOCATE_ARRAY(t, n)                \
+    ({                                               \
+        STATIC(TYPEOF_IS_SIZET(n));                  \
+        ASSERT_SIZE_MUL_NO_OVERFLOW((n), sizeof(t)); \
+        json_ast_allocate(ast, (n) * sizeof(t),      \
+            MEM_ALIGNOF(t[0]));                      \
+    })
+
+static const uchar_t* json_ast_new_delim(
+    struct json_ast_t* ast, const uchar_t* delim)
+{
+    uchar_t* d;
+    size_t n;
+
+    ASSERT(delim != NULL);
+    if (*delim == 0)
+        return (const uchar_t*) "";
+
+    n = strulen(delim);
+    ASSERT_SIZE_INC_NO_OVERFLOW(n);
+    d = JSON_AST_ALLOCATE_ARRAY(uchar_t, n + 1);
+
+    memcpy(d, delim, n + 1);
+    return d;
+}
+
 static struct json_ast_node_t* json_ast_new_node(
     struct json_ast_t* ast)
 {
@@ -191,12 +223,17 @@ static struct json_ast_node_t* json_ast_new_string_node(
     const struct json_text_pos_t* pos)
 {
     struct json_ast_node_t* n;
+    const uchar_t* d;
 
     n = json_ast_new_node(ast);
     ASSERT(n != NULL);
 
+    if ((d = json_get_token_delim(ast->json_obj)))
+         d = json_ast_new_delim(ast, d);
+
     n->type = json_ast_string_node_type;
     n->string.buf = *buf;
+    n->string.delim = d;
     n->pos = *pos;
 
     return n;
@@ -241,7 +278,7 @@ static bool json_ast_null(struct json_ast_t* ast)
     struct json_text_pos_t w;
     struct json_ast_node_t* n;
 
-    w = json_get_current_tok_pos(ast->json_obj);
+    w = json_get_token_pos(ast->json_obj);
     n = json_ast_new_null_node(ast, &w);
     JSON_AST_STACK_PUSH_NODE(n);
     return true;
@@ -253,31 +290,25 @@ static bool json_ast_boolean(struct json_ast_t* ast,
     struct json_text_pos_t w;
     struct json_ast_node_t* n;
 
-    w = json_get_current_tok_pos(ast->json_obj);
+    w = json_get_token_pos(ast->json_obj);
     n = json_ast_new_boolean_node(ast, val, &w);
     JSON_AST_STACK_PUSH_NODE(n);
     return true;
 }
 
-#define JSON_AST_ALLOCATE_ARRAY(t, n)                \
-    ({                                               \
-        STATIC(TYPEOF_IS_SIZET(n));                  \
-        ASSERT_SIZE_MUL_NO_OVERFLOW((n), sizeof(t)); \
-        json_ast_allocate(ast, (n) * sizeof(t),      \
-            MEM_ALIGNOF(t[0]));                      \
-    })
-
-#define JSON_AST_INIT_BUF(b, n)         \
-    do {                                \
+#define JSON_AST_MAKE_BUF(s, n)         \
+    ({                                  \
         uchar_t* __b;                   \
         ASSERT_SIZE_INC_NO_OVERFLOW(n); \
         __b = JSON_AST_ALLOCATE_ARRAY(  \
             uchar_t, (n) + 1);          \
-        memcpy(__b, val, (n));          \
+        memcpy(__b, (s), (n));          \
         __b[(n)] = 0;                   \
-        b.ptr = __b;                    \
-        b.size = (n);                   \
-    } while (0)
+        (struct json_ast_buf_t) {       \
+            .ptr = __b,                 \
+            .size = (n)                 \
+        };                              \
+    })
 
 static bool json_ast_number(
     struct json_ast_t* ast, const uchar_t* val, size_t len)
@@ -286,9 +317,9 @@ static bool json_ast_number(
     struct json_ast_node_t* n;
     struct json_ast_buf_t b;
 
-    JSON_AST_INIT_BUF(b, len);
+    b = JSON_AST_MAKE_BUF(val, len);
 
-    w = json_get_current_tok_pos(ast->json_obj);
+    w = json_get_token_pos(ast->json_obj);
     n = json_ast_new_number_node(ast, &b, &w);
     JSON_AST_STACK_PUSH_NODE(n);
 
@@ -302,9 +333,9 @@ static bool json_ast_string(
     struct json_ast_node_t* n;
     struct json_ast_buf_t b;
 
-    JSON_AST_INIT_BUF(b, len);
+    b = JSON_AST_MAKE_BUF(val, len);
 
-    w = json_get_current_tok_pos(ast->json_obj);
+    w = json_get_token_pos(ast->json_obj);
     n = json_ast_new_string_node(ast, &b, &w);
     JSON_AST_STACK_PUSH_NODE(n);
 
@@ -315,7 +346,7 @@ static bool json_ast_object_start(struct json_ast_t* ast)
 {
     struct json_text_pos_t w;
 
-    w = json_get_current_tok_pos(ast->json_obj);
+    w = json_get_token_pos(ast->json_obj);
     JSON_AST_STACK_PUSH_MARK(w);
     return true;
 }
@@ -386,7 +417,7 @@ static bool json_ast_array_start(struct json_ast_t* ast)
 {
     struct json_text_pos_t w;
 
-    w = json_get_current_tok_pos(ast->json_obj);
+    w = json_get_token_pos(ast->json_obj);
     JSON_AST_STACK_PUSH_MARK(w);
     return true;
 }
