@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018, 2019  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -17,6 +17,7 @@
 
 #include "config.h"
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -181,21 +182,12 @@ struct json_token_val_t
     };
 };
 
-#define json_token_pos_t json_error_pos_t
-
-static inline struct json_token_pos_t json_token_pos_offset(
-    struct json_token_pos_t pos, size_t col)
-{
-    ASSERT_SIZE_ADD_NO_OVERFLOW(pos.col, col);
-    pos.col += col;
-    return pos;
-}
-
 enum json_token_part_type_t {
     json_token_complete_part,
     json_token_literal_part,
     json_token_number_part,
-    json_token_string_part
+    json_token_string_part,
+    json_token_rstring_part
 };
 
 struct json_token_literal_part_t
@@ -310,6 +302,45 @@ struct json_token_string_part_t
     size_t pos;
 };
 
+enum json_token_rstring_state_t
+{
+    json_token_rstring_start_state,
+    json_token_rstring_quote_char_state,
+    json_token_rstring_open_delim_state,
+    json_token_rstring_plain_char_state,
+    json_token_rstring_close_delim_state,
+    json_token_rstring_error_state,
+};
+
+#ifdef JSON_DEBUG
+
+#undef  CASE_STATE
+#define CASE_STATE(n) [json_token_rstring_ ## n ## _state] = #n
+
+static const char* json_token_rstring_state_get_name(
+    enum json_token_rstring_state_t state)
+{
+    static const char* const names[] = {
+        CASE_STATE(start),
+        CASE_STATE(quote_char),
+        CASE_STATE(open_delim),
+        CASE_STATE(plain_char),
+        CASE_STATE(close_delim),
+        CASE_STATE(error)
+    };
+    return ARRAY_NON_NULL_ELEM(names, state);
+}
+
+#endif
+
+struct json_token_rstring_part_t
+{
+    enum json_token_rstring_state_t state;
+    uchar_t delim[16 + 1];
+    uchar_t* ptr;
+    size_t pos;
+};
+
 struct json_token_part_t
 {
     enum json_token_part_type_t type;
@@ -317,6 +348,7 @@ struct json_token_part_t
         struct json_token_literal_part_t literal;
         struct json_token_number_part_t  number;
         struct json_token_string_part_t  string;
+        struct json_token_rstring_part_t rstring;
     };
 };
 
@@ -351,12 +383,15 @@ struct json_token_part_t
         &(obj->current_tok.part.n);         \
     })
 
+#define json_token_pos_t json_error_pos_t
+
 struct json_token_t
 {
     enum json_token_type_t   type;
     struct json_token_val_t  val;
     struct json_token_pos_t  pos;
-    struct json_token_part_t part;                 
+    struct json_token_part_t part;
+    bits_t is_raw_str: 1;
 };
 
 #ifdef JSON_DEBUG
@@ -476,6 +511,29 @@ static void json_token_string_part_print_debug(
             val->pos);
 }
 
+static void json_token_rstring_part_print_debug(
+    const struct json_token_part_t* part)
+{
+    const struct json_token_rstring_part_t* val =
+        PART_VALUE(rstring);
+    enum { N = ARRAY_SIZE(val->delim) };
+    size_t n;
+
+    if (val->ptr == NULL)
+        n = 0;
+    else {
+        ASSERT(val->ptr >= val->delim);
+        ASSERT(val->ptr <= val->delim + N);
+        n = PTR_DIFF(val->ptr, val->delim);
+    }
+
+    print_debug_fmt(
+        "{.state=%s .delim=[%p]\"%*s\" .ptr=%p .pos=%zu}",
+            json_token_rstring_state_get_name(val->state),
+            val->delim, SIZE_AS_INT(n), val->delim,
+            val->ptr, val->pos);
+}
+
 #define CASE_PART2(n, f) [json_token_ ## n ## _part] = { .name = #n, .func = f }
 #define CASE_PART(n)     CASE_PART2(n, json_token_ ## n ## _part_print_debug)
 
@@ -490,14 +548,15 @@ static void json_token_part_print_debug(const struct json_token_part_t* part)
         CASE_PART2(complete, NULL),
         CASE_PART(literal),
         CASE_PART(number),
-        CASE_PART(string)
+        CASE_PART(string),
+        CASE_PART(rstring)
     };
     const struct part_info_t* p;
 
     print_debug_str("{.type=");
 
     if ((p = ARRAY_NULL_ELEM_REF(infos, part->type))) {
-        print_debug_fmt("%s", p->name);
+        print_debug_fmt("%s ", p->name);
         if (p->func)
             p->func(part);
     }
@@ -585,6 +644,7 @@ static inline void json_mem_buf_reset(struct json_mem_buf_t* buf)
 { mem_buf_update(buf, NULL, 0, mem_buf_reset); }
 
 #define JSON_MEM_BUF_APPEND(buf, ch) MEM_BUF_APPEND(buf, ch)
+#define JSON_MEM_BUF_REMOVE(buf, n)  MEM_BUF_REMOVE(buf, n)
 
 static enum json_code_point_type_t json_mem_buf_encode_utf8(
     struct json_mem_buf_t* buf,
@@ -652,7 +712,6 @@ static bool json_parse_literal_token(
     struct json_token_literal_part_t* part;
     size_t n = 0;
 
-    ASSERT(JSON_TOKEN_PART_TYPE_IS(literal));
     ASSERT(buf->ptr < buf->end);
 
     part = JSON_TOKEN_PART(literal);
@@ -686,18 +745,19 @@ static bool json_parse_literal_end_token(struct json_obj_t* obj)
 }
 
 #ifdef JSON_DEBUG
-#define PRINT_PARSE_TOKEN_BUF(n, w)                                           \
-    do {                                                                      \
-        PRINT_DEBUG_BEGIN("state=%-" #w "s ch=",                              \
-            json_token_ ## n ## _state_get_name(part->state));                \
-        s = pretty_print_char(stderr, ch, pretty_print_char_quotes);          \
-        ASSERT(s >= 3 && s <= 6);                                             \
-        PRINT_DEBUG_FMT("%-*s buf={.ptr=%p .end=%p ", SIZE_AS_INT(6 - s), "", \
-            buf->ptr, buf->end);                                              \
-        pretty_print_string(stderr, buf->ptr, PTR_DIFF(buf->end, buf->ptr),   \
-            pretty_print_string_quotes);                                      \
-        PRINT_DEBUG_STR("}");                                                 \
-        PRINT_DEBUG_END();                                                    \
+#define PRINT_PARSE_TOKEN_BUF(n, w)                                         \
+    do {                                                                    \
+        size_t __s;                                                         \
+        PRINT_DEBUG_BEGIN("state=%-" #w "s ch=",                            \
+            json_token_ ## n ## _state_get_name(part->state));              \
+        __s = pretty_print_char(stderr, ch, pretty_print_char_quotes);      \
+        ASSERT(__s >= 3 && __s <= 6);                                       \
+        PRINT_DEBUG_FMT("%-*s buf={.ptr=%p .end=%p ", SIZE_AS_INT(6 - __s), \
+            "", buf->ptr, buf->end);                                        \
+        pretty_print_string(stderr, buf->ptr, PTR_DIFF(buf->end, buf->ptr), \
+            pretty_print_string_quotes);                                    \
+        PRINT_DEBUG_STR("}");                                               \
+        PRINT_DEBUG_END();                                                  \
     } while (0)
 #else
 #define PRINT_PARSE_TOKEN_BUF(n, w) \
@@ -726,12 +786,7 @@ static bool json_parse_number_token(
     struct json_mem_buf_t* muf;
     bool done = false;
     size_t n = 0;
-#ifdef JSON_DEBUG
-    size_t s;
-#endif
 
-    ASSERT(JSON_TOKEN_TYPE_IS(number));
-    ASSERT(JSON_TOKEN_PART_TYPE_IS(number));
     ASSERT(buf->ptr < buf->end);
 
     val = JSON_TOKEN_VALUE(number);
@@ -880,9 +935,6 @@ static bool json_parse_number_end_token(struct json_obj_t* obj)
 {
     struct json_token_number_part_t* part;
 
-    ASSERT(JSON_TOKEN_TYPE_IS(number));
-    ASSERT(JSON_TOKEN_PART_TYPE_IS(number));
-
     part = JSON_TOKEN_PART(number);
 
     if (part->state == JSON_NUM_STATE(digitsn) ||
@@ -934,12 +986,7 @@ static bool json_parse_string_token(
     struct json_mem_buf_t* muf;
     bool done = false;
     size_t n = 0;
-#ifdef JSON_DEBUG
-    size_t s;
-#endif
 
-    ASSERT(JSON_TOKEN_TYPE_IS(string));
-    ASSERT(JSON_TOKEN_PART_TYPE_IS(string));
     ASSERT(buf->ptr < buf->end);
 
     val = JSON_TOKEN_VALUE(string);
@@ -1091,8 +1138,12 @@ loop_out:
 
         if (JSON_OBJECT_CONF(validate_utf8) &&
             !json_mem_buf_validate_utf8(muf, &pos)) {
-            struct json_token_pos_t err = json_token_pos_offset(
-                obj->current_tok.pos, SIZE_PRE_INC(pos));
+            struct json_token_pos_t err = obj->current_tok.pos;
+
+            ASSERT_SIZE_INC_NO_OVERFLOW(pos);
+            ASSERT_SIZE_ADD_NO_OVERFLOW(err.col, pos + 1);
+            err.col += pos + 1;
+
             return JSON_LEX_ERROR_POS(invalid_utf8_encoding, err);
         }
 
@@ -1113,6 +1164,164 @@ static bool json_parse_string_end_token(struct json_obj_t* obj)
     return JSON_LEX_ERROR(invalid_string_literal);
 }
 
+#define JSON_RSTR_STATE(n) (json_token_rstring_ ## n ## _state)
+
+static bool json_parse_rstring_token(
+    struct json_obj_t* obj, struct json_parse_buf_t* buf)
+{
+    // C++11's basic source character set without whitespaces
+    // (' ', '\t', '\v', '\n', '\f', and '\r') and without '\\'
+    static const bool is_delim[256] = {
+        ['0'...'9'] = true, ['a'...'z'] = true, ['A'...'Z'] = true,
+        ['_'] = true, ['{'] = true, ['}'] = true, ['['] = true,
+        [']'] = true, ['#'] = true, ['('] = true, [')'] = true,
+        ['<'] = true, ['>'] = true, ['%'] = true, [':'] = true,
+        [';'] = true, ['.'] = true, ['?'] = true, ['*'] = true,
+        ['+'] = true, ['-'] = true, ['/'] = true, ['^'] = true,
+        ['&'] = true, ['|'] = true, ['~'] = true, ['!'] = true,
+        ['='] = true, [','] = true, ['"'] = true, ['\''] = true,
+    };
+    struct json_token_rstring_part_t* part;
+    enum { N = ARRAY_SIZE(part->delim) };
+    struct json_mem_buf_token_val_t* val;
+    struct json_mem_buf_t* muf;
+    const uchar_t* ptr;
+    bool done = false;
+    size_t n = 0;
+
+    STATIC(ARRAY_SIZE(is_delim) == UCHAR_MAX + 1);
+
+    ASSERT(buf->ptr < buf->end);
+
+    val = JSON_TOKEN_VALUE(string);
+    part = JSON_TOKEN_PART(rstring);
+    muf = val->buf;
+    ptr = buf->ptr;
+
+    ASSERT(muf->size >= muf->len);
+
+    while (buf->ptr < buf->end && !done) {
+        uchar_t ch = *buf->ptr;
+
+        PRINT_PARSE_TOKEN_BUF(rstring, 10);
+
+        switch (part->state) {
+
+        case JSON_RSTR_STATE(start):
+            ASSERT(muf->len == 0);
+            ASSERT(ch == 'r');
+
+            part->state = JSON_RSTR_STATE(quote_char);
+            break;
+
+        case JSON_RSTR_STATE(quote_char):
+            if (ch != '"') {
+                part->state = JSON_RSTR_STATE(error);
+                goto loop_out;
+            }
+            part->state = JSON_RSTR_STATE(open_delim);
+            part->ptr = part->delim;
+            break;
+
+        case JSON_RSTR_STATE(open_delim):
+            ASSERT(part->ptr < part->delim + N);
+            if (!is_delim[ch] || ch == ')' ||
+                (part->ptr == part->delim + (N - 1)
+                    && ch != '(')) {
+                part->state = JSON_RSTR_STATE(error);
+                goto loop_out;
+            }
+            // => ptr <  delim + (N - 1) ||
+            //    ptr == delim + (N - 1) && ch == '('
+            if (ch != '(')
+                *part->ptr ++ = ch;
+            else {
+                part->state = JSON_RSTR_STATE(plain_char);
+                *part->ptr = 0;
+            }
+            // => ptr <= delim + (N - 1)
+            break;
+
+                             plain_char:
+        case JSON_RSTR_STATE(plain_char):
+            if (!ch || (!IS_ASCII(ch)
+                    && JSON_OBJECT_CONF(disallow_non_ascii))) {
+                part->state = JSON_RSTR_STATE(error);
+                goto loop_out;
+            }
+            if (ch == ')') {
+                part->state = JSON_RSTR_STATE(close_delim);
+                part->ptr = part->delim;
+            }
+            JSON_MEM_BUF_APPEND(muf, ch);
+            break;
+
+        case JSON_RSTR_STATE(close_delim):
+            if (*part->ptr == ch && ch)
+                part->ptr ++;
+            else
+            if (*part->ptr != 0 || ch != '"') {
+                part->state = JSON_RSTR_STATE(plain_char);
+                // stev: do not consume 'ch' yet, but...
+                goto plain_char;
+            }
+            else // *ptr == 0 && ch == '"'
+                done = true;
+            JSON_MEM_BUF_APPEND(muf, ch);
+            break;
+
+        default:
+            UNEXPECT_VAR("%d", part->state);
+        }
+
+        buf->ptr ++;
+        n ++;
+    }
+loop_out:
+
+    json_text_update_pos(&obj->current_pos, ptr, n);
+
+    if (done) {
+        size_t pos;
+
+        ASSERT(part->state == JSON_RSTR_STATE(close_delim));
+
+        n = PTR_DIFF(part->ptr, part->delim);
+        ASSERT_SIZE_ADD_NO_OVERFLOW(n, SZ(2));
+        n += 2;
+
+        if (JSON_OBJECT_CONF(validate_utf8) &&
+            !json_mem_buf_validate_utf8(muf, &pos)) {
+            struct json_token_pos_t err = obj->current_tok.pos;
+
+            ASSERT_SIZE_INC_NO_OVERFLOW(n);
+            // stev: account for the opening sequence 'r"' DELIM* '('
+            json_text_update_pos(&err, muf->ptr, SIZE_ADD(pos, n + 1));
+
+            return JSON_LEX_ERROR_POS(invalid_utf8_encoding, err);
+        }
+
+        // stev: remove the closing sequence
+        // ')' DELIM* '"' from token buffer:
+        JSON_MEM_BUF_REMOVE(muf, n);
+
+        JSON_MEM_BUF_APPEND(muf, CH(0));
+        obj->current_tok.part.type = json_token_complete_part;
+        return true;
+    }
+    if (part->state != JSON_RSTR_STATE(error) &&
+        buf->ptr >= buf->end)
+        return true;
+
+    return JSON_LEX_ERROR(invalid_rstring_literal);
+}
+
+static bool json_parse_rstring_end_token(struct json_obj_t* obj)
+{
+    ASSERT(JSON_TOKEN_PART_TYPE_IS(rstring));
+    return JSON_LEX_ERROR(invalid_rstring_literal);
+}
+
 static bool json_parse_part_token(
     struct json_obj_t* obj, struct json_parse_buf_t* buf)
 {
@@ -1127,6 +1336,8 @@ static bool json_parse_part_token(
         return json_parse_number_token(obj, buf);
     case json_token_string_part:
         return json_parse_string_token(obj, buf);
+    case json_token_rstring_part:
+        return json_parse_rstring_token(obj, buf);
     default:
         UNEXPECT_VAR("%d", type);
     }
@@ -1145,6 +1356,8 @@ static bool json_parse_end_token(struct json_obj_t* obj)
         return json_parse_number_end_token(obj);
     case json_token_string_part:
         return json_parse_string_end_token(obj);
+    case json_token_rstring_part:
+        return json_parse_rstring_end_token(obj);
     default:
         UNEXPECT_VAR("%d", type);
     }
@@ -1199,6 +1412,7 @@ parse_out:
         obj->current_tok.type = json_ ## n ## _token;          \
         obj->current_tok.pos = obj->current_pos;               \
         obj->current_tok.part.type = json_token_complete_part; \
+        obj->current_tok.is_raw_str = false;                   \
         obj->current_pos.col ++;                               \
         buf->ptr ++;                                           \
         true;                                                  \
@@ -1210,6 +1424,7 @@ parse_out:
         obj->current_tok.part.type = json_token_literal_part;              \
         obj->current_tok.part.literal.end = lit_ ## l + strlen(lit_ ## l); \
         obj->current_tok.part.literal.ptr = lit_ ## l;                     \
+        obj->current_tok.is_raw_str = false;                               \
         json_parse_literal_token(obj, buf);                                \
     })
 
@@ -1233,27 +1448,39 @@ parse_out:
         obj->current_tok.part.string.surrogates.high = 0; \
         obj->current_tok.part.string.surrogates.low = 0;  \
         obj->current_tok.part.string.pos = 0;             \
+        obj->current_tok.is_raw_str = false;              \
     })
 
-#define JSON_PARSE_LITERAL_BUF_PART_INIT(m, n)                            \
+#define JSON_PARSE_LITERAL_BUF_PART_INIT_RSTRING() \
+    ({                                             \
+        obj->current_tok.part.rstring.ptr = NULL;  \
+        obj->current_tok.part.rstring.pos = 0;     \
+        obj->current_tok.is_raw_str = true;        \
+    })
+
+#define JSON_PARSE_LITERAL_BUF_PART_INIT(m, p)                            \
     ({                                                                    \
-        obj->current_tok.part.type = json_token_ ## n ## _part;           \
-        obj->current_tok.part.n.state = json_token_ ## n ## _start_state; \
+        obj->current_tok.part.type = json_token_ ## p ## _part;           \
+        obj->current_tok.part.p.state = json_token_ ## p ## _start_state; \
         JSON_PARSE_LITERAL_BUF_PART_INIT_ ## m();                         \
     })
 
-#define JSON_PARSE_LITERAL_BUF_PART(m, n)                   \
+#define JSON_PARSE_LITERAL_BUF_PART(m, n, p)                \
     ({                                                      \
         obj->current_tok.type = json_ ## n ## _token;       \
         obj->current_tok.val.n.buf = &obj->current_tok_buf; \
         obj->current_tok.pos = obj->current_pos;            \
         json_mem_buf_reset(&obj->current_tok_buf);          \
-        JSON_PARSE_LITERAL_BUF_PART_INIT(m, n);             \
-        json_parse_ ## n ## _token(obj, buf);               \
+        JSON_PARSE_LITERAL_BUF_PART_INIT(m, p);             \
+        json_parse_ ## p ## _token(obj, buf);               \
     })
 
-#define JSON_PARSE_NUMBER() JSON_PARSE_LITERAL_BUF_PART(NUMBER, number)
-#define JSON_PARSE_STRING() JSON_PARSE_LITERAL_BUF_PART(STRING, string)
+#define JSON_PARSE_NUMBER() \
+    JSON_PARSE_LITERAL_BUF_PART(NUMBER, number, number)
+#define JSON_PARSE_STRING() \
+    JSON_PARSE_LITERAL_BUF_PART(STRING, string, string)
+#define JSON_PARSE_RSTRING() \
+    JSON_PARSE_LITERAL_BUF_PART(RSTRING, string, rstring)
 
 static bool json_parse_begin_token(
     struct json_obj_t* obj, struct json_parse_buf_t* buf)
@@ -1297,6 +1524,10 @@ static bool json_parse_begin_token(
         return JSON_PARSE_NUMBER();
     case '"':
         return JSON_PARSE_STRING();
+    case 'r':
+        if (JSON_OBJECT_CONF(allow_raw_strings))
+            return JSON_PARSE_RSTRING();
+        // FALLTHROUGH
     default:
         return JSON_LEX_ERROR(invalid_char);
     }
@@ -1726,7 +1957,7 @@ enum json_parse_status_t json_parse_done(struct json_obj_t* obj)
     if (JSON_STATUS_IS(ok))
         return JSON_PARSE_RETURN(json_do_parse(obj, NULL, 0, true));
     else
-         return JSON_PARSE_RETURN(JSON_CURRENT_STATUS());
+        return JSON_PARSE_RETURN(JSON_CURRENT_STATUS());
 }
 
 struct json_obj_t* json_obj_create(
@@ -1774,6 +2005,7 @@ bool json_config_get_param(
 {
     switch (param) {
     case json_allow_empty_input_config:
+    case json_allow_raw_strings_config:
     case json_allow_literal_value_config:
     case json_allow_multi_objects_config:
     case json_allow_surrogate_pairs_config:
@@ -1792,6 +2024,7 @@ void json_config_set_param(
 {
     switch (param) {
     case json_allow_empty_input_config:
+    case json_allow_raw_strings_config:
     case json_allow_literal_value_config:
     case json_allow_multi_objects_config:
     case json_allow_surrogate_pairs_config:
@@ -1879,6 +2112,7 @@ static const char* json_lex_error_get_name(
     CASE(invalid_literal);
     CASE(invalid_number_literal);
     CASE(invalid_string_literal);
+    CASE(invalid_rstring_literal);
     CASE(invalid_utf8_encoding);
     default:
         UNEXPECT_VAR("%d", err);
@@ -1901,6 +2135,8 @@ static const char* json_lex_error_get_desc(
         return "invalid number literal";
     case json_lex_error_invalid_string_literal:
         return "invalid string literal";
+    case json_lex_error_invalid_rstring_literal:
+        return "invalid raw string literal";
     case json_lex_error_invalid_utf8_encoding:
         return "invalid utf-8 encoding";
     default:
@@ -1977,12 +2213,19 @@ const char* json_error_info_get_desc(
     }
 }
 
-struct json_text_pos_t json_get_current_tok_pos(
+struct json_text_pos_t json_get_token_pos(
     struct json_obj_t* obj)
 {
     return obj->current_tok.pos;
 }
 
+const uchar_t* json_get_token_delim(struct json_obj_t* obj)
+{
+    return obj->current_tok.is_raw_str
+        ? obj->current_tok.part.rstring.delim
+        : NULL;
+}
+
 bool json_get_is_error(struct json_obj_t* obj)
 {
     return obj->error_info.type != json_error_none;
