@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018, 2019  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -27,8 +27,8 @@
 #endif
 
 #define JSON_VERSION_MAJOR 0
-#define JSON_VERSION_MINOR 11
-#define JSON_VERSION_PATCH 1
+#define JSON_VERSION_MINOR 12
+#define JSON_VERSION_PATCH 0 
 
 #define JSON_VERSION                 \
     (                                \
@@ -37,12 +37,19 @@
         JSON_VERSION_PATCH           \
     )
 
+#define VERSION_ID(n) n \
+    ## _VERSION_MAJOR.n \
+    ## _VERSION_MINOR.n \
+    ## _VERSION_PATCH
+
 #ifndef __GNUC__
 #error we need a GCC compiler
 #elif __GNUC__ >= 4
 #define JSON_API __attribute__ ((visibility("default")))
+#define JSON_WEAK_API __attribute__ ((visibility("default"), weak))
 #else
 #define JSON_API
+#define JSON_WEAK_API
 #endif
 
 #ifndef JSON_UNSIGNED_CHAR_DEFINED
@@ -127,6 +134,7 @@ JSON_API void json_obj_destroy(struct json_obj_t* obj);
 
 enum json_config_param_t {
     json_allow_empty_input_config,
+    json_allow_raw_strings_config,
     json_allow_literal_value_config,
     json_allow_multi_objects_config,
     json_allow_surrogate_pairs_config,
@@ -168,6 +176,11 @@ struct json_error_pos_t
 
 #define json_text_pos_t json_error_pos_t
 
+JSON_API struct json_text_pos_t json_get_token_pos(
+    struct json_obj_t*);
+JSON_API const uchar_t* json_get_token_delim(
+    struct json_obj_t*);
+
 struct json_file_info_t
 {
     const char* name;
@@ -210,6 +223,7 @@ struct json_ast_number_node_t
 struct json_ast_string_node_t
 {
     struct json_ast_buf_t buf;
+    const uchar_t* delim;
 };
 
 struct json_ast_object_arg_t
@@ -343,6 +357,12 @@ JSON_API enum json_parse_status_t json_type_parse(
 JSON_API enum json_parse_status_t json_type_parse_done(
     struct json_type_t* type);
 
+JSON_API struct json_text_pos_t json_type_get_token_pos(
+    struct json_type_t* type);
+
+JSON_API const uchar_t* json_type_get_token_delim(
+    struct json_type_t* type);
+
 JSON_API bool json_type_get_is_error(
     struct json_type_t*);
 JSON_API struct json_error_pos_t json_type_get_error_pos(
