@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018, 2019  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -52,6 +52,13 @@ void pool_alloc_done(
     free(alloc->pool);
 }
 
+void pool_alloc_clear(
+    struct pool_alloc_t* alloc)
+{
+    ASSERT_POOL_ALLOC_INVARIANTS(pool);
+    alloc->ptr = alloc->pool;
+}
+
 size_t pool_alloc_get_size(
     const struct pool_alloc_t* alloc)
 {
@@ -101,4 +108,35 @@ void* pool_alloc_allocate(
     return r;
 }
 
+// stev: note that the function below is not
+// the reverse of 'pool_alloc_allocate', since
+// the alignment computations above cannot be
+// reversed accurately: the new 'ptr' computed
+// below could be at a non-zero distance -- of
+// an amount less or equal than 'align' bytes
+// -- from the old 'ptr' as it was before the
+// call to 'pool_alloc_allocate'
+
+bool pool_alloc_deallocate(
+    struct pool_alloc_t* alloc,
+    size_t n_bytes)
+{
+    size_t s;
+
+    // stev: assert the invariants of the pool
+    ASSERT_POOL_ALLOC_INVARIANTS(pool);
+
+    if (n_bytes == 0)
+        n_bytes = 1;
+
+    // stev: the used space in the pool
+    s = PTR_DIFF(alloc->ptr, alloc->pool);
+
+    if (n_bytes > s)
+        return false;
+
+    alloc->ptr -= n_bytes;
+    return true;
+}
+
 
