@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018, 2019  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -21,6 +21,7 @@
 #include "lib/json.h"
 
 #include "int-traits.h"
+#include "json-filters.h"
 #include "common.h"
 
 struct mem_buf_t;
@@ -45,9 +46,13 @@ struct obj_json_base_t
 #endif
     bits_t fixed_size_buf: 1;
     bits_t echo_input: 1;
+    bits_t filter_err: 1;
+    bits_t json_err: 1;
+    bits_t no_input: 1;
 
     struct json_intf_t* json;
-    struct mem_buf_t*   buf;
+    struct json_filter_chain_t* filter;
+    struct mem_buf_t* buf;
 };
 
 void obj_json_base_init_obj(
@@ -74,6 +79,10 @@ void obj_json_base_init_sobj_type_lib(
     struct obj_json_base_t* this,
     const struct options_t* opts);
 
+void obj_json_base_init_filter_lib(
+    struct obj_json_base_t* this,
+    const struct options_t* opts);
+
 void obj_json_base_init_esc(
     struct obj_json_base_t* this,
     const struct options_t* opts);
@@ -84,6 +93,9 @@ int obj_json_base_run(struct obj_json_base_t* this);
 
 bool obj_json_base_check_error(struct obj_json_base_t* this);
 
+const uchar_t* obj_json_base_get_token_delim(
+    struct obj_json_base_t* this);
+
 struct json_obj_t* obj_json_base_get_obj(
     struct obj_json_base_t* this);
 
@@ -96,6 +108,9 @@ struct json_type_t* obj_json_base_get_type(
 struct json_type_lib_t* obj_json_base_get_type_lib(
     struct obj_json_base_t* this);
 
+struct json_filter_lib_t* obj_json_base_get_filter_lib(
+    struct obj_json_base_t* this);
+
 struct json_esc_t* obj_json_base_get_esc(
     struct obj_json_base_t* this);
 
